#!/bin/sh 

# (C)Copyright                                                    
# Sony Computer Entertainment, Inc.,                              
# Toshiba Corporation,                                            
# International Business Machines Corporation,                    
# 2005.  All rights reserved.                                

#
# Embed SPE ELF executable into PPE object file, and define a
# global pointer variable refering to the embedded file.
#
# Usage: embedspu [flags] symbol_name input_filename output_filename
#
#        input_filename:  SPE ELF executable to be embedded
#        output_filename: Resulting PPE object file
#        symbol_name:     Name of global pointer variable to be defined
#        flags:           GCC flags defining PPE object file format
#                         (e.g. -m32 or -m64)
#

# Determine location of dependent utilities
file=`basename "$0"`
dir=`dirname "$0"`
[ "${file#ppu-}" != "$file" ] && PREFIX=ppu-

PPU_GCC=${PREFIX}gcc
SPU_READELF=spu-readelf

which $PPU_GCC > /dev/null 2>&1
if [ $? -eq 0 ]
then
	:
elif [ -a ${dir}/$PPU_GCC ]
then
	PPU_GCC=${dir}/$PPU_GCC
elif [ -a /opt/sce/ppu/bin/$PPU_GCC ]
then
	PPU_GCC=/opt/sce/ppu/bin/$PPU_GCC
else
	echo Cannot find $PPU_GCC.
	exit 1
fi

which $SPU_READELF > /dev/null 2>&1
if [ $? -eq 0 ]
then
	:
elif [ -a ${dir}/../../spu/bin/$SPU_READELF ]
then
	SPU_READELF=${dir}/../../spu/bin/$SPU_READELF
elif [ -a /opt/sce/spu/bin/$SPU_READELF ]
then
	SPU_READELF=/opt/sce/spu/bin/$SPU_READELF
else
	echo Cannot find $SPU_READELF.
	exit 1
fi

# Argument parsing
SYMBOL=
INFILE=
OUTFILE=
FLAGS=

while [ -n "$1" ]; do
  case $1 in
    -*) FLAGS="${FLAGS} $1"
	shift ;;
    *)  if [ -z $SYMBOL ]; then
	  SYMBOL=$1
	elif [ -z $INFILE ]; then
	  INFILE=$1
	elif [ -z $OUTFILE ]; then
	  OUTFILE=$1
	fi
	shift ;;
  esac
done

if [ -z "$SYMBOL" -o -z "$INFILE" -o -z "$OUTFILE" ]; then
  echo "Usage: $0 [symbol_name] [input_filename] [output_filename]"
  exit 1
fi

if [ ! -e "$INFILE" ]; then
  echo "${INFILE}: File not found"
  exit 1
fi

# Temp file names
TMPOBJ1=`mktemp emXXXXXX` || exit 1
TMPOBJ2=`mktemp emXXXXXX` || { rm -f $TMPOBJ1; exit 1; }

# Build object file holding pointer to embedded section
${PPU_GCC} ${FLAGS} -x c -c -o ${TMPOBJ1} - <<EOF  || { rm -f $TMPOBJ1 $TMPOBJ2; exit 1; }
static const unsigned char __speelf__[]
  __attribute__((section(".rodata.speelf"), aligned(128))) = { };

static unsigned char __spetoe__[]
  __attribute__((section(".data.spetoe"), aligned(128))) = { };

typedef struct spe_program_handle {
        unsigned int    handle_size;
        const void      *elf_image;
        void            *toe_shadow;
} spe_program_handle_t;

spe_program_handle_t ${SYMBOL} = { sizeof (spe_program_handle_t),
                                   __speelf__, __spetoe__ };
EOF

# Build object file holding embedded section itself
${PPU_GCC} ${FLAGS} -x assembler-with-cpp -c -o ${TMPOBJ2} - <<EOF || { rm -f $TMPOBJ1 $TMPOBJ2; exit 1; }
.section .rodata.speelf,"a",@progbits
.p2align 7
.incbin "${INFILE}"

.section .data.spetoe,"aw",@progbits
.p2align 7
`${SPU_READELF} -s ${INFILE} | egrep ' _EAR_' | sort -k 2 | awk \
'{ \
	print "#ifdef _LP64"; \
	print ".quad " substr($8, 6); \
	print ".quad 0"; \
	print "#else"; \
	print ".int 0"; \
	print ".int " substr($8, 6); \
	print ".int 0"; \
	print ".int 0"; \
	print "#endif"; \
}'`
EOF

# Link both object files together
${PPU_GCC} ${FLAGS} -nostartfiles -nostdlib -Wl,-r -Wl,-x \
    -o ${OUTFILE} ${TMPOBJ1} ${TMPOBJ2}

# Clean up
rm -f ${TMPOBJ1} ${TMPOBJ2}

